// Aprs v1.00 : Programme M5Stack LoRa.
// Ecrit par Frank LOBA.
// Date création : 24/10/2022.
// Dernière mise à jour : 24/10/2022.


/**** INCLUSION DES FICHIERS EN-TETE ****/
#include <M5Stack.h>
#include <M5LoRa.h>
#include <M5ModemConfig.h>


/**** DEIFITION DES CONSTANTES ****/
#define LORA_SCK  18
#define LORA_NSS  5
#define LORA_MISO 19  
#define LORA_MOSI 23
#define LORA_RST  26
#define LORA_IRQ  36

#define REG_MODEM_PHY_CONFIG_3_ADRESS 0x26  // See SEMTECH SX1276/77/78/79 Datasheet Rev. 6 - January 2019, page 91.
#define REG_MODEM_PHY_CONFIG_3_VALUE  0xC

#define FREQUENCE         433775000L
#define SPREADING_FACTOR  12
#define SIGNAL_BANDWIDTH  125000L
#define CODING_RATE_4     5
#define SYNC_WORD         0x12


/**** DEFINITION DES VARIABLES GLOBALES ****/
Modem   Modem;


/**** DEFINITION DE LA FONCTION D'INITIALISATION ****/
void setup()
{
  // Initialisation du système.
  M5.begin();
  M5.Power.begin();

  // Affectation des broches.
  SPI.begin(LORA_SCK, LORA_MISO, LORA_MOSI, LORA_NSS);
  LoRa.setPins(LORA_NSS, LORA_RST, LORA_IRQ);

  // Défintion de la fréquence et démarrage du LoRa
  if(!LoRa.begin(FREQUENCE))
  {
    M5.Lcd.println("Echec demarrage LoRa !");
    while(1);
  }

  // Définition des paramètres système pour LoRa.
  LoRa.setSpreadingFactor(SPREADING_FACTOR);
  LoRa.setSignalBandwidth(SIGNAL_BANDWIDTH);
  LoRa.setCodingRate4(CODING_RATE_4);
  LoRa.enableCrc();
  LoRa.setSyncWord(SYNC_WORD);

  Modem.writeRegister(REG_MODEM_PHY_CONFIG_3_ADRESS, REG_MODEM_PHY_CONFIG_3_VALUE);
}


/**** DEFINITION DE LA FONCTION D'EXECUTION ****/
void loop()
{
  M5.update();

  // Réception des messages radio.  

  // Détection de la réception d'un paquet.
  int PacketSize = LoRa.parsePacket();
  if (PacketSize)
  {
    // Effacement de l'écran.
    M5.Lcd.clear();

    // Positionnement du curseur.
    M5.Lcd.setCursor(0, 0);

    // Définition de la taille des caractères.
    M5.Lcd.setTextSize(2);

    // Tant qu'il y a un caractère dans le tampon, le lire et l'afficher à l'écran.
    while (LoRa.available())
    {
      // Lecture du caractère.
      char Car = (char)LoRa.read();

      // Affichage à l'écran du caractère.
      M5.Lcd.print(Car);
    }
  }

  delay(20);
}
